<?php

namespace Lc\SovBundle\Definition\Field\Ticket;

use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ChoiceField;
use EasyCorp\Bundle\EasyAdminBundle\Field\DateField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;
use Lc\CaracoleBundle\Field\Filter\ProductCategoriesFilter;
use Lc\SovBundle\Definition\Field\AbstractFieldDefinition;
use Lc\SovBundle\Field\CollectionField;
use Lc\SovBundle\Field\Filter\Ticket\EmailFilter;
use Lc\SovBundle\Field\Filter\Ticket\FirstnameFilter;
use Lc\SovBundle\Field\Filter\Ticket\LastnameFilter;
use Lc\SovBundle\Form\Ticket\TicketMessageType;
use Lc\SovBundle\Solver\Ticket\TicketSolver;
use Lc\SovBundle\Translation\TranslatorAdmin;
use Symfony\Component\Form\Extension\Core\Type\TextType;

class TicketFieldDefinition extends AbstractFieldDefinition
{
    protected TicketSolver $ticketSolver;

    public function __construct(TranslatorAdmin $translatorAdmin, TicketSolver $ticketSolver)
    {
        parent::__construct($translatorAdmin);
        $this->ticketSolver = $ticketSolver;
    }

    public function configureIndex(): array
    {
        return [
                'id',
                'createdAt',
                'lastname',
                'firstname',
                'email',
                'type',
                'subject',
                'lastMessage',
                'status'
        ];
    }

    public function configureForm(): array
    {
        return [
                'user',
                'type',
                'subject',
                'ticketMessages'
        ];
    }

    public function configureFields(): array
    {
        return [
                'createdAt' => DateField::new('createdAt')->setFormat('short'),
                'visitorFirstname' => TextField::new('visitorFirstname'),
                'visitorLastname' => TextField::new('visitorLastname'),
                'visitorEmail' => TextField::new('visitorEmail'),
                'lastname' => TextField::new('lastname')
                        ->setTemplatePath('@LcSov/admin/ticket/field/lastname.html.twig')
                        ->setCustomOption('filter_fqcn', LastnameFilter::class),
                'firstname' => TextField::new('firstname')
                        ->setTemplatePath('@LcSov/admin/ticket/field/firstname.html.twig')
                        ->setCustomOption('filter_fqcn', FirstnameFilter::class),
                'email' => TextField::new('email')
                        ->setTemplatePath('@LcSov/admin/ticket/field/email.html.twig')
                        ->setCustomOption('filter_fqcn', EmailFilter::class),
                'user' => AssociationField::new('user'),
                'type' => ChoiceField::new('type')
                        ->autocomplete()
                        ->setChoices(
                                $this->translatorAdmin->transChoices(
                                        $this->ticketSolver->getTypeChoices(),
                                        'Ticket',
                                        'type'
                                )
                        ),
                'subject' => TextField::new('subject'),
                'lastMessage' => TextField::new('lastMessage')
                        ->setTemplatePath('@LcSov/admin/ticket/field/lastmessage.html.twig'),
                'status' => ChoiceField::new('status')
                        ->autocomplete()
                        ->setChoices(
                                $this->translatorAdmin->transChoices(
                                        $this->ticketSolver->getStatusChoices(),
                                        'Ticket',
                                        'status'
                                )

                        )
                        ->setTemplatePath('@LcSov/admin/ticket/field/status.html.twig'),
                'ticketMessages' => CollectionField::new('ticketMessages')
                        ->setFormTypeOption('entry_type', TicketMessageType::class)
                        ->setFormTypeOption('allow_add', false)
        ];
    }

}
